@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import enum
+import typing
 from dataclasses import dataclass
 from decimal import Decimal
 from typing import List, Optional, Union
@@ -323,47 +324,15 @@ class IntraAccountTransfer:
     def execute(cls, receiving_amount: Decimal, from_account: Union[CashAccount, ReserveAccount], to_account: Union[CashAccount, ReserveAccount], profile_type: ProfileTypes) -> "IntraAccountTransfer":
         user_profile: UserProfile = UserProfile.get_by_profile_type(profile_type)
 
+        if receiving_amount < Decimal("0"):
+            to_account, from_account = from_account, to_account
+            receiving_amount = receiving_amount * Decimal("-1")
+
         try:
             if isinstance(to_account, CashAccount):
-                if isinstance(from_account, ReserveAccount):
-                    wise_intra_account_transfer = wise_models.IntraAccountTransfer.call(user_profile.id, from_account.id, to_account.id, float(receiving_amount), None, to_account.currency.value)
-                else:
-                    wise_quote: wise_models.Quote = wise_models.Quote.call(user_profile.id, from_account.currency.value, to_account.currency.value, float(receiving_amount))
-                    wise_intra_account_transfer = wise_models.IntraAccountTransfer.call(user_profile.id, from_account.id, to_account.id, float(receiving_amount), wise_quote.id if isinstance(from_account, CashAccount) else None, None)
-
-                intra_account_transfer = IntraAccountTransfer(from_account, to_account, wise_intra_account_transfer)
-
-                if intra_account_transfer.is_successful:
-                    telegram.send_message(
-                        telegram.constants.telegram_channel_username,
-                                                        f"<u><b>Intra account money transferred</b></u>"
-                                                        f"\n\nAmount: <i>{to_account.currency.value} {global_common.get_formatted_string_from_decimal(receiving_amount)}</i>"
-                                                        f"\nFrom: <i>{from_account.name if isinstance(from_account, ReserveAccount) else to_account.currency.value}</i>"
-                                                        f"\nTo: <i>{to_account.name if isinstance(to_account, ReserveAccount) else to_account.currency.value}</i>", True)
-
-            elif isinstance(to_account, ReserveAccount):
-                if from_account.currency != to_account.currency:
-                    to_cash_account: CashAccount = CashAccount.get_by_profile_type_and_currency(profile_type, to_account.currency)
-                    IntraAccountTransfer.execute(receiving_amount, from_account, to_cash_account, profile_type)
-                    from_account = to_cash_account
-
-                if from_account.balance < receiving_amount:
-                    raise InsufficientFundsException(f"Insufficient funds"
-                                                     f"\nRequired amount: {from_account.currency.value} {global_common.get_formatted_string_from_decimal(receiving_amount)}"
-                                                     f"\nAccount Balance: {from_account.currency.value} {global_common.get_formatted_string_from_decimal(from_account.balance)}"
-                                                     f"\nShort of: {from_account.currency.value} {global_common.get_formatted_string_from_decimal(receiving_amount - from_account.balance)}")
-
-                wise_intra_account_transfer = wise_models.IntraAccountTransfer.call(user_profile.id, from_account.id, to_account.id, float(receiving_amount), None, to_account.currency.value)
-                intra_account_transfer = IntraAccountTransfer(from_account, to_account, wise_intra_account_transfer)
-
-                if intra_account_transfer.is_successful:
-                    telegram.send_message(
-                        telegram.constants.telegram_channel_username,
-                                                        f"<u><b>Intra account money transferred</b></u>"
-                                                        f"\n\nAmount: <i>{to_account.currency.value} {global_common.get_formatted_string_from_decimal(receiving_amount)}</i>"
-                                                        f"\nFrom: <i>{to_account.currency.value}</i>"
-                                                        f"\nTo: <i>{to_account.name if isinstance(to_account, ReserveAccount) else ''} ({to_account.currency.value})</i>", True)
-            return intra_account_transfer
+                return IntraAccountTransfer._transfer_to_cash_account(receiving_amount, from_account, to_account, user_profile)
+            else:
+                return IntraAccountTransfer._transfer_to_reserve_account(receiving_amount, typing.cast(CashAccount, from_account), to_account, user_profile)
 
         except ClientErrorException as e:
             logger.error(str(e))
@@ -382,5 +351,50 @@ class IntraAccountTransfer:
                                                 f"\nTo: <i>{to_account.name if isinstance(to_account, ReserveAccount) else to_account.currency.value} ({to_account.currency.value})</i>"
                                                 f"\nReason: <i>{str(e)}</i>", True)
             raise InsufficientFundsException(str(e))
-
         return None
+
+    @classmethod
+    def _transfer_to_cash_account(cls, receiving_amount: Decimal, from_account: Union[CashAccount, ReserveAccount], to_account: CashAccount, user_profile: UserProfile) -> "IntraAccountTransfer":
+        if isinstance(from_account, ReserveAccount):
+            wise_intra_account_transfer = wise_models.IntraAccountTransfer.call(user_profile.id, from_account.id, to_account.id, float(receiving_amount), None, to_account.currency.value)
+        else:
+            wise_quote: wise_models.Quote = wise_models.Quote.call(user_profile.id, from_account.currency.value,to_account.currency.value, float(receiving_amount))
+            wise_intra_account_transfer = wise_models.IntraAccountTransfer.call(user_profile.id, from_account.id, to_account.id, float(receiving_amount), wise_quote.id if isinstance(from_account, CashAccount) else None, None)
+
+        intra_account_transfer = IntraAccountTransfer(from_account, to_account, wise_intra_account_transfer)
+
+        if intra_account_transfer.is_successful:
+            telegram.send_message(
+                telegram.constants.telegram_channel_username,
+                f"<u><b>Intra account money transferred</b></u>"
+                f"\n\nAmount: <i>{to_account.currency.value} {global_common.get_formatted_string_from_decimal(receiving_amount)}</i>"
+                f"\nFrom: <i>{from_account.name if isinstance(from_account, ReserveAccount) else to_account.currency.value}</i>"
+                f"\nTo: <i>{to_account.currency.value}</i>", True)
+
+        return intra_account_transfer
+
+    @classmethod
+    def _transfer_to_reserve_account(cls, receiving_amount: Decimal, from_account: CashAccount, to_account: ReserveAccount, user_profile: UserProfile) -> "IntraAccountTransfer":
+        if from_account.currency != to_account.currency:
+            to_cash_account: CashAccount = CashAccount.get_by_profile_type_and_currency(user_profile.type, to_account.currency)
+            IntraAccountTransfer.execute(receiving_amount, from_account, to_cash_account, user_profile.type)
+            from_account = to_cash_account
+
+        if from_account.balance < receiving_amount:
+            raise InsufficientFundsException(f"Insufficient funds"
+                                             f"\nRequired amount: {from_account.currency.value} {global_common.get_formatted_string_from_decimal(receiving_amount)}"
+                                             f"\nAccount Balance: {from_account.currency.value} {global_common.get_formatted_string_from_decimal(from_account.balance)}"
+                                             f"\nShort of: {from_account.currency.value} {global_common.get_formatted_string_from_decimal(receiving_amount - from_account.balance)}")
+
+        wise_intra_account_transfer = wise_models.IntraAccountTransfer.call(user_profile.id, from_account.id, to_account.id, float(receiving_amount), None, to_account.currency.value)
+        intra_account_transfer = IntraAccountTransfer(from_account, to_account, wise_intra_account_transfer)
+
+        if intra_account_transfer.is_successful:
+            telegram.send_message(
+                telegram.constants.telegram_channel_username,
+                f"<u><b>Intra account money transferred</b></u>"
+                f"\n\nAmount: <i>{to_account.currency.value} {global_common.get_formatted_string_from_decimal(receiving_amount)}</i>"
+                f"\nFrom: <i>{to_account.currency.value}</i>"
+                f"\nTo: <i>{to_account.name if isinstance(to_account, ReserveAccount) else ''} ({to_account.currency.value})</i>", True)
+
+        return intra_account_transfer
\ No newline at end of file
